package com.mortals.xhx.module.certificate.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.common.code.ClassifyType;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.certificate.dao.CertificateCatalogDao;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateChildEntity;
import com.mortals.xhx.module.certificate.model.CertificateChildQuery;
import com.mortals.xhx.module.certificate.model.CertificateClassifyEntity;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateChildService;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * CertificateCatalogService
 * 证照目录 service实现
 *
 * @author zxfei
 * @date 2022-10-14
 */
@Service("certificateCatalogService")
public class CertificateCatalogServiceImpl extends AbstractCRUDServiceImpl<CertificateCatalogDao, CertificateCatalogEntity, Long> implements CertificateCatalogService {

    @Value("${upload.path}")
    private String filePath;

    @Value("${upload.url:http://192.168.0.98:17216/cpm/file/commonupload?prePath=/file/uploadfile}")
    private String uploadUrl;

    @Value("${qrcode.url:http://rhpt.scjg.chengdu.gov.cn:8091/adap/spjy/process/generateQrCode.do?dto.applyId=}")
    private String qrcode;

    @Autowired
    private CertificateClassifyService certificateClassifyService;

    @Autowired
    private CertificateChildService certificateChildService;


    @Override
    protected void saveBefore(CertificateCatalogEntity entity, Context context) throws AppException {
        if (StringUtils.isEmpty(entity.getCatalogName())) {
            throw new AppException("目录名称不能为空");
        }
        if (StringUtils.isEmpty(entity.getCatalogCode())) {
            throw new AppException("目录编码不能为空");
        }
        if (entity.getHolderType() == null) {
            throw new AppException("持有者类型不能为空");
        }
        if (entity.getClassifyId() == null) {
            throw new AppException("目录分类不能为空");
        }
        if (entity.getIndustryId() == null) {
            throw new AppException("所属行业不能为空");
        }

    }

    @Override
    protected void updateBefore(CertificateCatalogEntity entity, Context context) throws AppException {
        if (StringUtils.isEmpty(entity.getCatalogName())) {
            throw new AppException("目录名称不能为空");
        }
        if (StringUtils.isEmpty(entity.getCatalogCode())) {
            throw new AppException("目录编码不能为空");
        }
        if (entity.getHolderType() == null) {
            throw new AppException("持有者类型不能为空");
        }
        if (entity.getClassifyId() == null) {
            throw new AppException("目录分类不能为空");
        }
        if (entity.getIndustryId() == null) {
            throw new AppException("所属行业不能为空");
        }

    }


    @Override
    protected CertificateCatalogEntity findBefore(CertificateCatalogEntity params, PageInfo pageInfo, Context context) throws AppException {
        if (StringUtils.isNotEmpty(params.getCatalogName())) {
            params.setCatalogName("%".concat(params.getCatalogName()).concat("%"));
        }
        return params;
    }

    @Override
    protected void findAfter(CertificateCatalogEntity params, PageInfo pageInfo, Context context, List<CertificateCatalogEntity> list) throws AppException {
        if (CollectionUtils.isNotEmpty(list)) {
            for (CertificateCatalogEntity entity : list) {
                CertificateClassifyEntity certificateClassifyEntity = certificateClassifyService.get(entity.getClassifyId());
                if (certificateClassifyEntity != null) {
                    entity.setClassifyType(certificateClassifyEntity.getClassifyType());
                } else {
                    entity.setClassifyType(ClassifyType.PRINCIPAL.getValue());
                }
            }
        }
        fillSubData(list);
    }

    @Override
    protected void findAfter(CertificateCatalogEntity params, Context context, List<CertificateCatalogEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<CertificateCatalogEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        CertificateChildQuery certificateChildQuery = new CertificateChildQuery();
        certificateChildQuery.setCatalogIdList(idList);
        Map<Long, List<CertificateChildEntity>> certificateChildListMap = certificateChildService.find(certificateChildQuery).stream().collect(Collectors.groupingBy(CertificateChildEntity::getCatalogId));
        list.forEach(item -> item.setCertificateChildList(certificateChildListMap.get(item.getId())));
    }

    @Override
    protected void saveAfter(CertificateCatalogEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getCertificateChildList())) {
            entity.getCertificateChildList().stream().peek(item -> {
                item.setCatalogId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            certificateChildService.save(entity.getCertificateChildList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(CertificateCatalogEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getCertificateChildList())) {
            Long[] certificateChildIds = certificateChildService.find(new CertificateChildQuery().catalogId(entity.getId())).stream().map(CertificateChildEntity::getId).toArray(Long[]::new);
            certificateChildService.remove(certificateChildIds, context);
            entity.getCertificateChildList().stream().peek(item -> {
                item.setCatalogId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            certificateChildService.save(entity.getCertificateChildList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<CertificateChildEntity> certificateChildlist = certificateChildService.find(new CertificateChildQuery().catalogIdList(Arrays.asList(ids)));
        certificateChildService.removeList(certificateChildlist, context);
        super.removeAfter(ids, context, result);
    }


    public static void main(String[] args) {
//        String excelFilePath = "D:/mortals/app/data/cpm/" + "excel/test1.xls";
//        List<String> excelList = new ArrayList<>();
//        excelList.add("i_1_姓名");
//        excelList.add("i_2_性别");
//        excelList.add("i_3_年龄");
//        excelList.add("i_4_证件编号");
//        excelList.add("i_5_证件名称");
//
//        ExcelUtil.createCatalogTemplate(excelFilePath,excelList);
    }

    @Override
    public void generateExcel(Long id, Context context) {
//        CertificateCatalogEntity entity = dao.get(id);
//        parseDocxToJson(entity,true);
//        CertificateCatalogEntity updata = new CertificateCatalogEntity();
//        updata.setId(entity.getId());
//        updata.setExcelFile(entity.getExcelFile());
//        updata.setUpdateTime(new Date());
//        dao.update(updata);
    }
}