package com.mortals.xhx.module.child.service.impl;
import com.mortals.xhx.common.code.ProcessStatusEnum;
import com.mortals.xhx.module.certificate.model.CertificateChildEntity;
import com.mortals.xhx.module.certificate.model.CertificateChildQuery;
import com.mortals.xhx.module.certificate.model.CertificateDocumentEntity;
import com.mortals.xhx.module.certificate.model.CertificateDocumentQuery;
import com.mortals.xhx.module.certificate.service.CertificateChildService;
import com.mortals.xhx.module.certificate.service.CertificateDocumentService;
import com.mortals.xhx.module.record.model.ApplyLogEntity;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.child.dao.ChildLicenseDao;
import com.mortals.xhx.module.child.model.ChildLicenseEntity;
import com.mortals.xhx.module.child.service.ChildLicenseService;
import lombok.extern.slf4j.Slf4j;
/**
* ChildLicenseService
* 行业许可子证 service实现
*
* @author zxfei
* @date 2024-07-28
*/
@Service("childLicenseService")
@Slf4j
public class ChildLicenseServiceImpl extends AbstractCRUDServiceImpl<ChildLicenseDao, ChildLicenseEntity, Long> implements ChildLicenseService {

    @Autowired
    private CertificateChildService certificateChildService;
    @Autowired
    private CertificateDocumentService certificateDocumentService;
    @Override
    public int createChildLicense(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        int result = 0;
        if(applyLogEntity.getCatalogId()!=null) {
            List<CertificateChildEntity> childEntityList = certificateChildService.find(new CertificateChildQuery().catalogId(applyLogEntity.getCatalogId()));
            List<Long> documentIdList = childEntityList.stream().map(CertificateChildEntity::getDocumentId).collect(Collectors.toList());
            List<CertificateDocumentEntity> documentEntityList = certificateDocumentService.find(new CertificateDocumentQuery().idList(documentIdList));
            if (CollectionUtils.isNotEmpty(documentEntityList)) {
                List<ChildLicenseEntity> childLicense = new ArrayList<>();
                for (CertificateDocumentEntity item : documentEntityList) {
                    ChildLicenseEntity entity = new ChildLicenseEntity();
                    entity.initAttrValue();
                    entity.setSiteId(applyLogEntity.getSiteId());
                    entity.setCatalogId(applyLogEntity.getCatalogId());
                    entity.setCatalogName(applyLogEntity.getCatalogName());
                    entity.setApplyId(applyLogEntity.getId());
                    entity.setDocumentId(item.getId());
                    entity.setDocumentName(item.getDocumentName());
                    entity.setDeptId(item.getDeptId());
                    entity.setDeptName(item.getDeptName());
                    entity.setMarketEntityName(applyLogEntity.getEnterpriseName());
                    entity.setLicenseCode(applyLogEntity.getCertificateCode());
                    entity.setLegalPersonName(applyLogEntity.getLegalPerson());
                    entity.setCreditCode(applyLogEntity.getSocialCode());
                    entity.setProductLicenseTime(applyLogEntity.getIssueTime());
                    entity.setCertQRCode(applyLogEntity.getQRCode());
                    entity.setCertAuthority(applyLogEntity.getAuthority());
                    entity.setBusinessPlace(applyLogEntity.getBusinessPlace());
                    entity.setLicensedItems(applyLogEntity.getLicenseProject());
                    entity.setFormStyleContent(item.getTemplateContent());
                    entity.setProcessStatus(ProcessStatusEnum.未处理.getValue());
                    entity.setCreateTime(applyLogEntity.getCreateTime());
                    entity.setCreateUserId(applyLogEntity.getCreateUserId());
                    childLicense.add(entity);
                }
                this.dao.insertBatch(childLicense);
            }
        }
        return result;
    }
}